@@ -1,8 +1,7 @@
-import os, os.path as osp, logging, glob, shutil
+import os, os.path as osp, logging, glob, shutil, time
 import qondor
 logger = logging.getLogger('qondor')
 
-
 class CMSSW(object):
     """docstring for CMSSW"""
 
@@ -25,6 +24,12 @@ class CMSSW(object):
     @classmethod
     def from_tarball(cls, tarball, scram_arch=None, outdir=None):
         cmssw_src = cls.extract_tarball(tarball, outdir)
+        # See if the tarball was already compiled with some scram_arch, if so use it
+        if scram_arch is None:
+            compiled_arches = glob.glob(osp.join(cmssw_src, '../bin/slc*'))
+            if compiled_arches:
+                scram_arch = osp.basename(compiled_arches[0])
+                logger.warning('Detected tarball was compiled with arch %s, using it', scram_arch)
         return cls(cmssw_src, scram_arch)
 
     @classmethod
@@ -74,11 +79,21 @@ class CMSSW(object):
         super(CMSSW, self).__init__()
         self.cmssw_src = osp.abspath(cmssw_src)
         if scram_arch is None:
-            logger.warning(
-                'Taking SCRAM_ARCH from environment; may mismatch with'
-                ' CMSSW version of %s', self.cmssw_src
-                )
-            self.scram_arch = os.environ['SCRAM_ARCH']
+            compiled_arches = glob.glob(osp.join(self.cmssw_src, '../bin/slc*'))
+            if compiled_arches:
+                self.scram_arch = osp.basename(compiled_arches[0])
+                logger.warning('Detected CMSSW was compiled with arch %s, using it', self.scram_arch)
+            else:
+                try:
+                    logger.warning('Attempting to find scram_arch...')
+                    print(osp.basename(osp.normpath(osp.join(self.cmssw_src,'..'))))
+                    self.scram_arch = qondor.get_arch(osp.basename(osp.normpath(osp.join(self.cmssw_src,'..'))))
+                except RuntimeError:
+                    logger.warning(
+                        'Taking SCRAM_ARCH from environment; may mismatch with'
+                        ' CMSSW version of %s', self.cmssw_src
+                        )
+                    self.scram_arch = os.environ['SCRAM_ARCH']
         else:
             self.scram_arch = scram_arch
         self._is_renamed = False
@@ -96,6 +111,34 @@ class CMSSW(object):
         logger.info('Doing scram b ExternalLinks %s', self.cmssw_src)
         self.run_commands_nocmsenv(['scram b ExternalLinks'])
 
+    def cmsrun(self, cfg_file, **kwargs):
+        """
+        Specifically runs a cmsRun command. Expects a python file, and other
+        keyword arguments are parsed as "keyword=value".
+        The key "inputFiles" is treated differently: A list is expected,
+        and every item is added as "inputFiles=item".
+        The key "outputFile" is also treated differently: The passed string
+        is formatted with keys "proc_id", "cluster_id", and "datestr" for
+        convenience.
+        """
+        cmd =  [
+            'cmsRun', cfg_file
+            ]
+        if 'inputFiles' in kwargs:
+            inputFiles = kwargs.pop('inputFiles')
+            if qondor.utils.is_string(inputFiles): inputFiles = [inputFiles]
+            for inputFile in inputFiles:
+                cmd.append('inputFiles='+inputFile)
+        if 'outputFile' in kwargs:
+            outputFile = kwargs.pop('outputFile')
+            cmd.append('outputFile=' + outputFile.format(
+                proc_id = qondor.get_proc_id(),
+                cluster_id = qondor.get_cluster_id(),
+                datestr = time.strftime('%b%d')
+                ))
+        cmd.extend([ '{0}={1}'.format(k,v) for k, v in kwargs.items() ])
+        self.run_command(cmd)
+
     def run_command(self, cmd):
         """
         Mostly legacy; use run_commands instead
@@ -125,7 +168,11 @@ class CMSSW(object):
                 env = qondor.utils.get_clean_env()
                 )
 
-    def make_tarball(self, outdir='.', tag=None, renew=True):
+    def make_tarball(
+        self, outdir='.', tag=None, renew=True,
+        exclude=None, exclude_vcs=True, exclude_caches_all=True,
+        include=None,
+        ):
         """
         Makes a tarball out of the CMSSW distribution of this class
         """
@@ -149,14 +196,18 @@ class CMSSW(object):
             'Tarballing {0} ==> {1}'
             .format(cmssw_path, dst)
             )
+        if exclude is None: exclude = ['tmp']
         with qondor.utils.switchdir(osp.dirname(cmssw_path)):
             cmd = [
                 'tar',
-                '--exclude-caches-all',
-                '--exclude-vcs',
                 '-zcvf',
                 dst,
-                osp.basename(cmssw_path),
-                '--exclude=tmp',
                 ]
+            if include:
+                if qondor.utils.is_string(include): include = [include]
+                cmd.extend(include)
+            if exclude_vcs: cmd.append('--exclude-vcs',)
+            if exclude_caches_all: cmd.append('--exclude-caches-all',)
+            cmd.extend(['--exclude={0}'.format(e) for e in exclude])
+            cmd.append(osp.basename(cmssw_path))
             qondor.utils.run_command(cmd)
